classdef FrameRigidBody < sequence.Frame 
    %sequence.FrameRigidBody Represents the information of a single frame of the video
    %sequence: image data, ground truth information, sequence order, etc.
    %
    %  The ground truth information stores rigid body transformations
    %  (rotation + translation) in R3.
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2014-04-10
    %
    %  @lastmod 2014-04-10
    %
    %  @changelog 2014-04-10  enrique.munoz@iit.it
    %
    %             Initial version
    %

    properties 
        %% Properties
        
        % Parameter values for the first Euler angle
        Alpha
        
        % Parameter values for the second Euler angle
        Beta
        
        % Parameter values for the third Euler angle
        Gamma
        
        % Parameter values for the first translation component
        Tx
        
        % Parameter values for the second translation component
        Ty
        
        % Parameter values for the third translation component
        Tz
    
        
    end% properties
    
    methods
        %% Methods
        
        %
        function obj = FrameRigidBody(frame) ;
            %% Default constructor
                        
            % The Frame object is initialized with a frame number (order in
            % the sequence)  
            obj = obj@sequence.Frame(frame) ;
            
            % Six parameters as we are dealing with rigid body
            % transformations
            obj.NumParameters = 6 ;

            % The name of the parameters: 3 corresponding to the rotation
            % (Euler angles alpha, beta, and gamma)
            obj.ParametersNames = {'Alpha','Beta','Gamma','Tx','Ty','Tz'} ;

        
        end % sequence.Frame constructor
    
        %
        function obj = assign(obj, params)
            % Assigns all the parameters of the Frame to the values in
            % params. 
            obj.Alpha = params(1) ;
            obj.Beta  = params(2) ;
            obj.Gamma = params(3) ;
            obj.Tx    = params(4) ;
            obj.Ty    = params(5) ;
            obj.Tz    = params(6) ;
            
        end% sequence.Frame.assign
                
        %
        function params = vec(obj)
            % Retrieve the frame parameters in vector form
            params = [obj.Alpha;...
                      obj.Beta;...
                      obj.Gamma;...
                      obj.Tx;...
                      obj.Ty;...
                      obj.Tz] ;
            
        end% sequence.Frame.assign
                

       
    end% methods
    
end