classdef SequenceVICON < top.Sequence
    % sequence.SequenceVICON Loads the sequence data from a VICON capture
    % session. This data comprises images (video from the captured
    % sequence) as well as the ground truth (motion parameters estimated
    % with the VICON system). 
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2014-06-15
    %
    %  @lastmod 2014-06-15
    %
    %  @changelog 2014-06-15  enrique.munoz@iit.it
    %
    %             Initial version

    %
    properties
        %% Properties
        
        % Object that loads data from a VICON sequence
        DataLoader
        
        % 3D Positions of the VICON markers on the calibration plate.
        ReferencePoints
        
        % Origin of the object coordinate system in the calibration 
        % plate reference system.
        Origin
        
        % Transformation that aligns the object coordinate system to the
        % calibration plate reference system.
        MotionAlign = object.transform.RigidBody(eye(3), [0;0;0]) ;
        
    end
    
    %
    methods
        %% Methods

        %
        function obj = SequenceVICON(source, directory, name, objectClass) 
            %  
            
            % Call the superclass constructor  
            obj            = obj@top.Sequence(name) ;
            
            % Load data from VICON
            obj.DataLoader = vicon.DataLoaderVICONEdges(directory,...
                                                        source,...
                                                        'jpg') ;

            %
            obj.Configuration = obj.DataLoader.Configuration ;
                                                    
            %
            obj.NumFrames  = obj.DataLoader.NumFrames ;
            
            % Fill the Frames property with a list of frames
            for frame = 1:obj.NumFrames
               
                % Each frame is an object of class 'objectClass' that is
                % stored in its specific sequence order.
                Frames(frame) = feval(objectClass, frame) ;
                
            end
            
            % Assign the generated frames to the property Frames of the
            % object
            obj.Frames    = Frames ;

        end
            
        %
        function obj = process(obj)
            
            %
            obj.Configuration.dir.capture  = obj.DataLoader.Configuration.dir.frames ;
            
            %
            obj.Extension                  = obj.DataLoader.FrameFileExtension ;

            %
            obj.ViewPoint                  = obj.DataLoader.ViewPoint ; 
            
            %
            Torigin                        = obj.Origin - obj.ModelOrigin ;

            % Move the origin to the barycenter of the object
            obj.ReferencePoints            = obj.ReferencePoints - repmat(Torigin, 1, length(obj.ReferencePoints)) ;

            % Find the transformation between the marker points (the
            % trajectories) captured by VICON and the reference system on
            % the calibrated grid
            points                         = obj.DataLoader.GroundTruth(1).Trajectory ;

            % Compute the rigid transform between the first frame of the
            % sequence and the calibration grid references.
            [~,~,transform]                = procrustes(points',...
                                                        obj.ReferencePoints',...
                                                        'Scaling',false) ;
                                                
            % Extract motion parameters
            Rcurrent                       = (transform.T)' ;
            tcurrent                       = transform.c ;
            tcurrent                       = tcurrent(1,:)' ;
                
            % Store data
%             obj.CalibrationModel.Motion = object.transform.RigidBody(Rcurrent,tcurrent) ;
            
            %
            transformAlign                = object.transform.RigidBody(Rcurrent,tcurrent) ;

            %
            for frame = 1:obj.NumFrames
                
                % Load captured motion
                obj.Frames(frame)        = obj.Frames(frame).assign(obj.DataLoader.GroundTruth(frame)) ;
                                        
                % Update the captured motion such that it is expressed in
                % the coordinate system of the calibration plate.
                obj.Frames(frame).Motion = obj.Frames(frame).Motion*transformAlign ;
                
            end
            
        end % 
        
        %
        function obj = name(obj)
            
            % Copy sequence frames
            for frame= 1:obj.NumFrames
                
                %
                obj.Frames(frame).ImageName = obj.DataLoader.Frames(frame).Camera2.Name ;
            
            end
            
        end
        
        %
        function obj = init(obj)
            %% Create the actual output and temporary directories
            
            % Call to super-class method init().
            obj       = init@top.Sequence(obj) ;

        end
        
        %
        function value = Frame(obj, frame)
           
             % Complete file name corresponding to the current frame
             value = [obj.Configuration.dir.frames, filesep,...
                      obj.Frames(frame).ImageName,...
                      '.',obj.Extension] ;
            
        end
        
    end% methods
    
end