classdef Sequence < matlab.mixin.Copyable & dynamicprops
    %sequence.Sequence Represents an video sequence as a set of frames. The
    %Sequence object not only contains the video information (images) but
    %also the grount truth that generated those images.
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2014-04-10
    %
    %  @lastmod 2014-04-10
    %
    %  @changelog 2014-04-10  enrique.munoz@iit.it
    %
    %             Initial version
    %

    %
    properties
        %% Properties
        
        % Name of the sequence
        Name
        
        % Generated keyframes
        Frames

        % Number of frames
        NumFrames 

        % Configuration
        Configuration
        
        % Camera configuration
        ViewPoint
        
        % Image renderer
        Renderer
        
        % Default file extension
        Extension = 'png'
            
    end% Properties
    
    %
    properties (Dependent)
        %% Dependent properties
        
        % Object model
        Model
        
        % Origin of the model reference system.
        ModelOrigin
        
        % Root directory for the sequence
        DirectorySequence
        
        % Directory where the images are stored
        DirectoryFrames
        
        % Directory where the PoV-Ray files are stored
        DirectoryPovray
        
        % Extension suffix for the image files
        FileExtension 
        
        % Frames that are going to be rendered. By default, the whole set
        % of sequence frames.
        FramesToRender
        
        % Name of the parameters
        ParametersNames
        
        % Number of motion parameters
        NumParameters
        
    end % Dependent properties
        
    %
    methods
       %% Member functions
       
       %
       function obj = Sequence(name)
           
           % Name of the sequence (it will be used for creating the output directories)
           obj.Name                          = name ;
           
           % Create (if necessary) output directories for the sequence.
           obj.init() ;
           
       end
       
       %
       function obj = run(obj)
           %% Travels the set of Procedures, executing them one-by-one in the stored order.
                      
           % Create the images in disk
           obj.Renderer.render() ;
           
           % Save data to disk
           obj.save() ;

       end % run
        
       %
       function obj = generate(obj)
           %% Travels the set of Procedures, executing them one-by-one in the stored order.
                      
           % Create the images in disk
           obj.process() ;
           
           % Save data to disk
           obj.name() ;

       end % run
        
       %
       function obj = init(obj)
           %% Create the actual output and temporary directories
           
           
       end

       %
       function save(obj)
           
           %% Name of the data file
           file_name = [obj.DirectorySequence, filesep, obj.Name];

           %
           sequence  = obj ;
           
           % Save the actual file to disk
           save(file_name, 'sequence') ;

       end
       
       %
       function currentFrame = frame(obj, numFrame) 
       
           % Load the image of the current frame
           currentFrame           = obj.Frames(numFrame) ;
           
           %
           currentFrame.ImageData = imread(obj.Frame(numFrame)) ;
           
       end
       
       %
       function obj = name(obj)
          
           %
           for frame = 1:obj.NumFrames
  
               % File name for the current frame
               filename                    = [obj.Name,'.frame',...
                                              sprintf('%05d',frame),'.',obj.Extension] ;
                           
               % Load the current frame name into the data structure.
               obj.Frames(frame).ImageName = filename ;
  
           end
           
       end
       
       %
       function obj = add(obj, object)
           
           % Load model data from file name and store in the object
           filename                 = [object,'.mat'] ;
           s                        = load(filename) ;
           obj.Configuration.model  = s.model ;
           
       end
       
       %
       function value = get.ModelOrigin(obj)
           %
           value = obj.Configuration.model.origin ;
           
       end
               
       %
       function value = get.Model(obj)
           %
           value = [obj.Configuration.model.vertices;...
                    ones(1,size(obj.Configuration.model.vertices,2))] ;
           
       end
               
    end % methods
    
end
